# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2004-2005,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Cedar Backup, release 3
# Purpose  : Tests knapsack functionality.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Unit tests for CedarBackup3/knapsack.py.

Code Coverage
=============

   This module contains individual tests for each of the public functions
   implemented in knapsack.py: ``firstFit()``, ``bestFit()``, ``worstFit()`` and
   ``alternateFit()``.

   Note that the tests for each function are pretty much identical and so
   there's pretty much code duplication.  In production code, I would argue
   that this implies some refactoring is needed.  In here, however, I prefer
   having lots of individual test cases even if there is duplication, because I
   think this makes it easier to judge the extent of a problem when one exists.

Naming Conventions
==================

   I prefer to avoid large unit tests which validate more than one piece of
   functionality, and I prefer to avoid using overly descriptive (read: long)
   test names, as well.  Instead, I use lots of very small tests that each
   validate one specific thing.  These small tests are then named with an index
   number, yielding something like ``testAddDir_001`` or ``testValidate_010``.
   Each method has a docstring describing what it's supposed to accomplish.  I
   feel that this makes it easier to judge how important a given failure is,
   and also makes it somewhat easier to diagnose and fix individual problems.

Full vs. Reduced Tests
======================

   All of the tests in this module are considered safe to be run in an average
   build environment.  There is a no need to use a KNAPSACKTESTS_FULL
   environment variable to provide a "reduced feature set" test suite as for
   some of the other test modules.

@author Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Import modules and do runtime validations
########################################################################

# Import standard modules
import unittest

from CedarBackup3.knapsack import alternateFit, bestFit, firstFit, worstFit
from CedarBackup3.testutil import configureLogging

#######################################################################
# Module-wide configuration and constants
#######################################################################

# These all have random letters for keys because the original data had a,b,c,d,
# etc. in ascending order, which actually masked a sorting bug in the implementation.

ITEMS_01 = {}
ITEMS_02 = {
    "z": 0,
    "^": 0,
    "3": 0,
    "(": 0,
    "[": 0,
    "/": 0,
    "a": 0,
    "r": 0,
}
ITEMS_03 = {
    "k": 0,
    "*": 1,
    "u": 10,
    "$": 100,
    "h": 1000,
    "?": 10000,
    "b": 100000,
    "s": 1000000,
}
ITEMS_04 = {
    "l": 1000000,
    "G": 100000,
    "h": 10000,
    "#": 1000,
    "a": 100,
    "'": 10,
    "c": 1,
    "t": 0,
}
ITEMS_05 = {
    "n": 1,
    "N": 1,
    "z": 1,
    "@": 1,
    "c": 1,
    "h": 1,
    "d": 1,
    "u": 1,
}
ITEMS_06 = {
    "o": 10,
    "b": 10,
    "G": 10,
    "+": 10,
    "B": 10,
    "O": 10,
    "e": 10,
    "v": 10,
}
ITEMS_07 = {
    "$": 100,
    "K": 100,
    "f": 100,
    "=": 100,
    "n": 100,
    "I": 100,
    "F": 100,
    "w": 100,
}
ITEMS_08 = {
    "y": 1000,
    "C": 1000,
    "s": 1000,
    "f": 1000,
    "a": 1000,
    "U": 1000,
    "g": 1000,
    "x": 1000,
}
ITEMS_09 = {
    "7": 10000,
    "d": 10000,
    "f": 10000,
    "g": 10000,
    "t": 10000,
    "l": 10000,
    "h": 10000,
    "y": 10000,
}
ITEMS_10 = {
    "5": 100000,
    "#": 100000,
    "l": 100000,
    "t": 100000,
    "6": 100000,
    "T": 100000,
    "i": 100000,
    "z": 100000,
}
ITEMS_11 = {
    "t": 1,
    "d": 1,
    "k": 100000,
    "l": 100000,
    "7": 100000,
    "G": 100000,
    "j": 1,
    "1": 1,
}
ITEMS_12 = {
    "a": 10,
    "e": 10,
    "M": 100000,
    "u": 100000,
    "y": 100000,
    "f": 100000,
    "k": 10,
    "2": 10,
}
ITEMS_13 = {
    "n": 100,
    "p": 100,
    "b": 100000,
    "i": 100000,
    "$": 100000,
    "/": 100000,
    "l": 100,
    "3": 100,
}
ITEMS_14 = {
    "b": 1000,
    ":": 1000,
    "e": 100000,
    "O": 100000,
    "o": 100000,
    "#": 100000,
    "m": 1000,
    "4": 1000,
}
ITEMS_15 = {
    "c": 1,
    "j": 1,
    "e": 1,
    "H": 100000,
    "n": 100000,
    "h": 1,
    "N": 1,
    "5": 1,
}
ITEMS_16 = {
    "a": 10,
    "M": 10,
    "%": 10,
    "'": 100000,
    "l": 100000,
    "?": 10,
    "o": 10,
    "6": 10,
}
ITEMS_17 = {
    "h": 100,
    "z": 100,
    "(": 100,
    "?": 100000,
    "k": 100000,
    "|": 100,
    "p": 100,
    "7": 100,
}
ITEMS_18 = {
    "[": 1000,
    "l": 1000,
    "*": 1000,
    "/": 100000,
    "z": 100000,
    "|": 1000,
    "q": 1000,
    "h": 1000,
}

# This is a more realistic example, taken from tree9.tar.gz
ITEMS_19 = {
    "dir001/file001": 243,
    "dir001/file002": 268,
    "dir002/file001": 134,
    "dir002/file002": 74,
    "file001": 155,
    "file002": 242,
    "link001": 0,
    "link002": 0,
}


#######################################################################
# Utility functions
#######################################################################


def buildItemDict(origDict):
    """
    Creates an item dictionary suitable for passing to a knapsack function.

    The knapsack functions take a dictionary, keyed on item, of (item, size)
    tuples.  This function converts a simple item/size dictionary to a knapsack
    dictionary.  It exists for convenience.

    Args:
       origDict (Simple dictionary mapping item to size, like ``ITEMS_02``): Dictionary to convert
    Returns:
        Dictionary suitable for passing to a knapsack function
    """
    itemDict = {}
    for key in list(origDict.keys()):
        itemDict[key] = (key, origDict[key])
    return itemDict


#######################################################################
# Test Case Classes
#######################################################################

#####################
# TestKnapsack class
#####################


class TestKnapsack(unittest.TestCase):
    """Tests for the various knapsack functions."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    def setUp(self):
        pass

    def tearDown(self):
        pass

    ################################
    # Tests for firstFit() function
    ################################

    def testFirstFit_001(self):
        """
        Test firstFit() behavior for an empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 0
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testFirstFit_002(self):
        """
        Test firstFit() behavior for an empty items dictionary, non-zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 10000
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testFirstFit_003(self):
        """
        Test firstFit() behavior for an non-empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_04)
        capacity = 0
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_13)
        capacity = 0
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testFirstFit_004(self):
        """
        Test firstFit() behavior for non-empty items dictionary with zero-sized items, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testFirstFit_005(self):
        """
        Test firstFit() behavior for items dictionary where only one item fits.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 1
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 10
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 100
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 10000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 100000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100000, result[1])

    def testFirstFit_006(self):
        """
        Test firstFit() behavior for items dictionary where only 25% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 2
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 25
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 250
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 2500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 25000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 250000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 2
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 25
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 250
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 2500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

    def testFirstFit_007(self):
        """
        Test firstFit() behavior for items dictionary where only 50% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 4
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 45
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 450
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 4500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 45000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 450000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 4
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 45
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 450
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 4500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testFirstFit_008(self):
        """
        Test firstFit() behavior for items dictionary where only 75% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 6
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 65
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 650
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 6500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 65000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 650000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 7
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 65
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 650
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 6500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

    def testFirstFit_009(self):
        """
        Test firstFit() behavior for items dictionary where all items individually
        exceed the capacity.
        """
        items = buildItemDict(ITEMS_06)
        capacity = 9
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_07)
        capacity = 99
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_08)
        capacity = 999
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_09)
        capacity = 9999
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_10)
        capacity = 99999
        result = firstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testFirstFit_010(self):
        """
        Test firstFit() behavior for items dictionary where first half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 200
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testFirstFit_011(self):
        """
        Test firstFit() behavior for items dictionary where middle half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 5
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 50
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 5000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testFirstFit_012(self):
        """
        Test firstFit() behavior for items dictionary where second half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 200
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testFirstFit_013(self):
        """
        Test firstFit() behavior for items dictionary where first half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 50
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testFirstFit_014(self):
        """
        Test firstFit() behavior for items dictionary where middle half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 3
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_12)
        capacity = 35
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_13)
        capacity = 350
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_14)
        capacity = 3500
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testFirstFit_015(self):
        """
        Test firstFit() behavior for items dictionary where second half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 50
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testFirstFit_016(self):
        """
        Test firstFit() behavior for items dictionary where all items fit.
        """
        items = buildItemDict(ITEMS_02)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(0, result[1])

        items = buildItemDict(ITEMS_03)
        capacity = 2000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_04)
        capacity = 2000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_05)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400004, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400040, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(404000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200006, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200060, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 1000000
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(206000, result[1])

    def testFirstFit_017(self):
        """
        Test firstFit() behavior for a more realistic set of items
        """
        items = buildItemDict(ITEMS_19)
        capacity = 760
        result = firstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        # Unfortunately, can't test any more than this, since dict keys come out in random order

    ###############################
    # Tests for bestFit() function
    ###############################

    def testBestFit_001(self):
        """
        Test bestFit() behavior for an empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 0
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testBestFit_002(self):
        """
        Test bestFit() behavior for an empty items dictionary, non-zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 10000
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testBestFit_003(self):
        """
        Test bestFit() behavior for an non-empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_04)
        capacity = 0
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_13)
        capacity = 0
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testBestFit_004(self):
        """
        Test bestFit() behavior for non-empty items dictionary with zero-sized items, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testBestFit_005(self):
        """
        Test bestFit() behavior for items dictionary where only one item fits.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 1
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 10
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 100
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 10000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 100000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100000, result[1])

    def testBestFit_006(self):
        """
        Test bestFit() behavior for items dictionary where only 25% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 2
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 25
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 250
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 2500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 25000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 250000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 2
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 25
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 250
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 2500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

    def testBestFit_007(self):
        """
        Test bestFit() behavior for items dictionary where only 50% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 4
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 45
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 450
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 4500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 45000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 450000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 4
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 45
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 450
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 4500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testBestFit_008(self):
        """
        Test bestFit() behavior for items dictionary where only 75% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 6
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 65
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 650
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 6500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 65000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 650000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 7
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 65
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 650
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 6500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

    def testBestFit_009(self):
        """
        Test bestFit() behavior for items dictionary where all items individually
        exceed the capacity.
        """
        items = buildItemDict(ITEMS_06)
        capacity = 9
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_07)
        capacity = 99
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_08)
        capacity = 999
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_09)
        capacity = 9999
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_10)
        capacity = 99999
        result = bestFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testBestFit_010(self):
        """
        Test bestFit() behavior for items dictionary where first half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 200
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testBestFit_011(self):
        """
        Test bestFit() behavior for items dictionary where middle half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 5
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 50
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 5000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testBestFit_012(self):
        """
        Test bestFit() behavior for items dictionary where second half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 200
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testBestFit_013(self):
        """
        Test bestFit() behavior for items dictionary where first half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 50
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testBestFit_014(self):
        """
        Test bestFit() behavior for items dictionary where middle half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 3
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_12)
        capacity = 35
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_13)
        capacity = 350
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_14)
        capacity = 3500
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testBestFit_015(self):
        """
        Test bestFit() behavior for items dictionary where second half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 50
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testBestFit_016(self):
        """
        Test bestFit() behavior for items dictionary where all items fit.
        """
        items = buildItemDict(ITEMS_02)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(0, result[1])

        items = buildItemDict(ITEMS_03)
        capacity = 2000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_04)
        capacity = 2000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_05)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400004, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400040, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(404000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200006, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200060, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 1000000
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(206000, result[1])

    def testBestFit_017(self):
        """
        Test bestFit() behavior for a more realistic set of items
        """
        items = buildItemDict(ITEMS_19)
        capacity = 760
        result = bestFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(5, len(result[0]))
        self.assertEqual(753, result[1])
        self.assertTrue("dir001/file001" in result[0])
        self.assertTrue("dir001/file002" in result[0])
        self.assertTrue("file002" in result[0])
        self.assertTrue("link001" in result[0])
        self.assertTrue("link002" in result[0])

    ################################
    # Tests for worstFit() function
    ################################

    def testWorstFit_001(self):
        """
        Test worstFit() behavior for an empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 0
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testWorstFit_002(self):
        """
        Test worstFit() behavior for an empty items dictionary, non-zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 10000
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testWorstFit_003(self):
        """
        Test worstFit() behavior for an non-empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_04)
        capacity = 0
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_13)
        capacity = 0
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testWorstFit_004(self):
        """
        Test worstFit() behavior for non-empty items dictionary with zero-sized items, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testWorstFit_005(self):
        """
        Test worstFit() behavior for items dictionary where only one item fits.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 1
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 10
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 100
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 10000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 100000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100000, result[1])

    def testWorstFit_006(self):
        """
        Test worstFit() behavior for items dictionary where only 25% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 2
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 25
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 250
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 2500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 25000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 250000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 2
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 25
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 250
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 2500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

    def testWorstFit_007(self):
        """
        Test worstFit() behavior for items dictionary where only 50% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 4
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 45
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 450
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 4500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 45000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 450000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 4
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 45
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 450
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 4500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testWorstFit_008(self):
        """
        Test worstFit() behavior for items dictionary where only 75% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 6
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 65
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 650
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 6500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 65000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 650000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 7
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 65
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 650
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 6500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

    def testWorstFit_009(self):
        """
        Test worstFit() behavior for items dictionary where all items individually
        exceed the capacity.
        """
        items = buildItemDict(ITEMS_06)
        capacity = 9
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_07)
        capacity = 99
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_08)
        capacity = 999
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_09)
        capacity = 9999
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_10)
        capacity = 99999
        result = worstFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testWorstFit_010(self):
        """
        Test worstFit() behavior for items dictionary where first half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 200
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testWorstFit_011(self):
        """
        Test worstFit() behavior for items dictionary where middle half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 5
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 50
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 5000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testWorstFit_012(self):
        """
        Test worstFit() behavior for items dictionary where second half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 200
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testWorstFit_013(self):
        """
        Test worstFit() behavior for items dictionary where first half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 50
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testWorstFit_014(self):
        """
        Test worstFit() behavior for items dictionary where middle half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 3
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_12)
        capacity = 35
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_13)
        capacity = 350
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_14)
        capacity = 3500
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testWorstFit_015(self):
        """
        Test worstFit() behavior for items dictionary where second half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 50
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testWorstFit_016(self):
        """
        Test worstFit() behavior for items dictionary where all items fit.
        """
        items = buildItemDict(ITEMS_02)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(0, result[1])

        items = buildItemDict(ITEMS_03)
        capacity = 2000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_04)
        capacity = 2000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_05)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400004, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400040, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(404000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200006, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200060, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 1000000
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(206000, result[1])

    def testWorstFit_017(self):
        """
        Test worstFit() behavior for a more realistic set of items
        """
        items = buildItemDict(ITEMS_19)
        capacity = 760
        result = worstFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(605, result[1])
        self.assertTrue("dir002/file001" in result[0])
        self.assertTrue("dir002/file002" in result[0])
        self.assertTrue("file001" in result[0])
        self.assertTrue("file002" in result[0])
        self.assertTrue("link001" in result[0])
        self.assertTrue("link002" in result[0])

    ####################################
    # Tests for alternateFit() function
    ####################################

    def testAlternateFit_001(self):
        """
        Test alternateFit() behavior for an empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 0
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testAlternateFit_002(self):
        """
        Test alternateFit() behavior for an empty items dictionary, non-zero capacity.
        """
        items = buildItemDict(ITEMS_01)
        capacity = 10000
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testAlternateFit_003(self):
        """
        Test alternateFit() behavior for an non-empty items dictionary, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_04)
        capacity = 0
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_13)
        capacity = 0
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testAlternateFit_004(self):
        """
        Test alternateFit() behavior for non-empty items dictionary with zero-sized items, zero capacity.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 0
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testAlternateFit_005(self):
        """
        Test alternateFit() behavior for items dictionary where only one item fits.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 1
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 10
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 100
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(1000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 10000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(10000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 100000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(1, len(result[0]))
        self.assertEqual(100000, result[1])

    def testAlternateFit_006(self):
        """
        Test alternateFit() behavior for items dictionary where only 25% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 2
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 25
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 250
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 2500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 25000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 250000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 2
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 25
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(20, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 250
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(200, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 2500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(2, len(result[0]))
        self.assertEqual(2000, result[1])

    def testAlternateFit_007(self):
        """
        Test alternateFit() behavior for items dictionary where only 50% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 4
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 45
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 450
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 4500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 45000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 450000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 4
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 45
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 450
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 4500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testAlternateFit_008(self):
        """
        Test alternateFit() behavior for items dictionary where only 75% of items fit.
        """
        items = buildItemDict(ITEMS_05)
        capacity = 6
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 65
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 650
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 6500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 65000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 650000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 7
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 65
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(60, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 650
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 6500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(6000, result[1])

    def testAlternateFit_009(self):
        """
        Test alternateFit() behavior for items dictionary where all items individually
        exceed the capacity.
        """
        items = buildItemDict(ITEMS_06)
        capacity = 9
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_07)
        capacity = 99
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_08)
        capacity = 999
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_09)
        capacity = 9999
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

        items = buildItemDict(ITEMS_10)
        capacity = 99999
        result = alternateFit(items, capacity)
        self.assertEqual(([], 0), result)

    def testAlternateFit_010(self):
        """
        Test alternateFit() behavior for items dictionary where first half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 200
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testAlternateFit_011(self):
        """
        Test alternateFit() behavior for items dictionary where middle half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 5
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 50
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(40, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 5000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(4000, result[1])

    def testAlternateFit_012(self):
        """
        Test alternateFit() behavior for items dictionary where second half of items
        individually exceed capacity and remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 200
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(4, len(result[0]))
        self.assertEqual(111, result[1])

    def testAlternateFit_013(self):
        """
        Test alternateFit() behavior for items dictionary where first half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_04)
        capacity = 50
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testAlternateFit_014(self):
        """
        Test alternateFit() behavior for items dictionary where middle half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_11)
        capacity = 3
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_12)
        capacity = 35
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_13)
        capacity = 350
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

        items = buildItemDict(ITEMS_14)
        capacity = 3500
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testAlternateFit_015(self):
        """
        Test alternateFit() behavior for items dictionary where second half of items
        individually exceed capacity and only some of remainder fit.
        """
        items = buildItemDict(ITEMS_03)
        capacity = 50
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertTrue(len(result[0]) < 4, "%s < 4" % len(result[0]))

    def testAlternateFit_016(self):
        """
        Test alternateFit() behavior for items dictionary where all items fit.
        """
        items = buildItemDict(ITEMS_02)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(0, result[1])

        items = buildItemDict(ITEMS_03)
        capacity = 2000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_04)
        capacity = 2000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(1111111, result[1])

        items = buildItemDict(ITEMS_05)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8, result[1])

        items = buildItemDict(ITEMS_06)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80, result[1])

        items = buildItemDict(ITEMS_07)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800, result[1])

        items = buildItemDict(ITEMS_08)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(8000, result[1])

        items = buildItemDict(ITEMS_09)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(80000, result[1])

        items = buildItemDict(ITEMS_10)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(800000, result[1])

        items = buildItemDict(ITEMS_11)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400004, result[1])

        items = buildItemDict(ITEMS_12)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400040, result[1])

        items = buildItemDict(ITEMS_13)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(400400, result[1])

        items = buildItemDict(ITEMS_14)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(404000, result[1])

        items = buildItemDict(ITEMS_15)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200006, result[1])

        items = buildItemDict(ITEMS_16)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200060, result[1])

        items = buildItemDict(ITEMS_17)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(200600, result[1])

        items = buildItemDict(ITEMS_18)
        capacity = 1000000
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(8, len(result[0]))
        self.assertEqual(206000, result[1])

    def testAlternateFit_017(self):
        """
        Test alternateFit() behavior for a more realistic set of items
        """
        items = buildItemDict(ITEMS_19)
        capacity = 760
        result = alternateFit(items, capacity)
        self.assertTrue(result[1] <= capacity, "%s <= %s" % (result[1], capacity))
        self.assertEqual(6, len(result[0]))
        self.assertEqual(719, result[1])
        self.assertTrue("link001" in result[0])
        self.assertTrue("dir001/file002" in result[0])
        self.assertTrue("link002" in result[0])
        self.assertTrue("dir001/file001" in result[0])
        self.assertTrue("dir002/file002" in result[0])
        self.assertTrue("dir002/file001" in result[0])



# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2007,2010,2013,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Official Cedar Backup Extensions
# Purpose  : Provides an extension to split up large files in staging directories.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Provides an extension to split up large files in staging directories.

When this extension is executed, it will look through the configured Cedar
Backup staging directory for files exceeding a specified size limit, and split
them down into smaller files using the 'split' utility.  Any directory which
has already been split (as indicated by the ``cback.split`` file) will be
ignored.

This extension requires a new configuration section <split> and is intended
to be run immediately after the standard stage action or immediately before the
standard store action.  Aside from its own configuration, it requires the
options and staging configuration sections in the standard Cedar Backup
configuration file.

:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""

########################################################################
# Imported modules
########################################################################

import logging
import os
import re
from functools import total_ordering

from CedarBackup3.actions.util import findDailyDirs, getBackupFiles, writeIndicatorFile
from CedarBackup3.config import ByteQuantity, addByteQuantityNode, readByteQuantity
from CedarBackup3.util import changeOwnership, executeCommand, resolveCommand
from CedarBackup3.xmlutil import addContainerNode, createInputDom, readFirstChild

########################################################################
# Module-wide constants and variables
########################################################################

logger = logging.getLogger("CedarBackup3.log.extend.split")

SPLIT_COMMAND = ["split"]
SPLIT_INDICATOR = "cback.split"


########################################################################
# SplitConfig class definition
########################################################################


@total_ordering
class SplitConfig(object):
    """
    Class representing split configuration.

    Split configuration is used for splitting staging directories.

    The following restrictions exist on data in this class:

       - The size limit must be a ByteQuantity
       - The split size must be a ByteQuantity

    """

    def __init__(self, sizeLimit=None, splitSize=None):
        """
        Constructor for the ``SplitCOnfig`` class.

        Args:
           sizeLimit: Size limit of the files, in bytes
           splitSize: Size that files exceeding the limit will be split into, in bytes

        Raises:
           ValueError: If one of the values is invalid
        """
        self._sizeLimit = None
        self._splitSize = None
        self.sizeLimit = sizeLimit
        self.splitSize = splitSize

    def __repr__(self):
        """
        Official string representation for class instance.
        """
        return "SplitConfig(%s, %s)" % (self.sizeLimit, self.splitSize)

    def __str__(self):
        """
        Informal string representation for class instance.
        """
        return self.__repr__()

    def __eq__(self, other):
        """Equals operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) == 0

    def __lt__(self, other):
        """Less-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) < 0

    def __gt__(self, other):
        """Greater-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) > 0

    def __cmp__(self, other):
        """
        Original Python 2 comparison operator.
        Lists within this class are "unordered" for equality comparisons.
        Args:
           other: Other object to compare to
        Returns:
            -1/0/1 depending on whether self is ``<``, ``=`` or ``>`` other
        """
        if other is None:
            return 1
        if self.sizeLimit != other.sizeLimit:
            if (self.sizeLimit or ByteQuantity()) < (other.sizeLimit or ByteQuantity()):
                return -1
            else:
                return 1
        if self.splitSize != other.splitSize:
            if (self.splitSize or ByteQuantity()) < (other.splitSize or ByteQuantity()):
                return -1
            else:
                return 1
        return 0

    def _setSizeLimit(self, value):
        """
        Property target used to set the size limit.
        If not ``None``, the value must be a ``ByteQuantity`` object.
        Raises:
           ValueError: If the value is not a ``ByteQuantity``
        """
        if value is None:
            self._sizeLimit = None
        else:
            if not isinstance(value, ByteQuantity):
                raise ValueError("Value must be a ``ByteQuantity`` object.")
            self._sizeLimit = value

    def _getSizeLimit(self):
        """
        Property target used to get the size limit.
        """
        return self._sizeLimit

    def _setSplitSize(self, value):
        """
        Property target used to set the split size.
        If not ``None``, the value must be a ``ByteQuantity`` object.
        Raises:
           ValueError: If the value is not a ``ByteQuantity``
        """
        if value is None:
            self._splitSize = None
        else:
            if not isinstance(value, ByteQuantity):
                raise ValueError("Value must be a ``ByteQuantity`` object.")
            self._splitSize = value

    def _getSplitSize(self):
        """
        Property target used to get the split size.
        """
        return self._splitSize

    sizeLimit = property(_getSizeLimit, _setSizeLimit, None, doc="Size limit, as a ByteQuantity")
    splitSize = property(_getSplitSize, _setSplitSize, None, doc="Split size, as a ByteQuantity")


########################################################################
# LocalConfig class definition
########################################################################


@total_ordering
class LocalConfig(object):
    """
    Class representing this extension's configuration document.

    This is not a general-purpose configuration object like the main Cedar
    Backup configuration object.  Instead, it just knows how to parse and emit
    split-specific configuration values.  Third parties who need to read and
    write configuration related to this extension should access it through the
    constructor, ``validate`` and ``addConfig`` methods.

    *Note:* Lists within this class are "unordered" for equality comparisons.

    """

    def __init__(self, xmlData=None, xmlPath=None, validate=True):
        """
        Initializes a configuration object.

        If you initialize the object without passing either ``xmlData`` or
        ``xmlPath`` then configuration will be empty and will be invalid until it
        is filled in properly.

        No reference to the original XML data or original path is saved off by
        this class.  Once the data has been parsed (successfully or not) this
        original information is discarded.

        Unless the ``validate`` argument is ``False``, the :any:`LocalConfig.validate`
        method will be called (with its default arguments) against configuration
        after successfully parsing any passed-in XML.  Keep in mind that even if
        ``validate`` is ``False``, it might not be possible to parse the passed-in
        XML document if lower-level validations fail.

        *Note:* It is strongly suggested that the ``validate`` option always be set
        to ``True`` (the default) unless there is a specific need to read in
        invalid configuration from disk.

        Args:
           xmlData (String data): XML data representing configuration
           xmlPath (Absolute path to a file on disk): Path to an XML file on disk
           validate (Boolean true/false): Validate the document after parsing it
        Raises:
           ValueError: If both ``xmlData`` and ``xmlPath`` are passed-in
           ValueError: If the XML data in ``xmlData`` or ``xmlPath`` cannot be parsed
           ValueError: If the parsed configuration document is not valid
        """
        self._split = None
        self.split = None
        if xmlData is not None and xmlPath is not None:
            raise ValueError("Use either xmlData or xmlPath, but not both.")
        if xmlData is not None:
            self._parseXmlData(xmlData)
            if validate:
                self.validate()
        elif xmlPath is not None:
            with open(xmlPath) as f:  # pylint: disable=unspecified-encoding
                xmlData = f.read()
            self._parseXmlData(xmlData)
            if validate:
                self.validate()

    def __repr__(self):
        """
        Official string representation for class instance.
        """
        return "LocalConfig(%s)" % (self.split)

    def __str__(self):
        """
        Informal string representation for class instance.
        """
        return self.__repr__()

    def __eq__(self, other):
        """Equals operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) == 0

    def __lt__(self, other):
        """Less-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) < 0

    def __gt__(self, other):
        """Greater-than operator, iplemented in terms of original Python 2 compare operator."""
        return self.__cmp__(other) > 0

    def __cmp__(self, other):
        """
        Original Python 2 comparison operator.
        Lists within this class are "unordered" for equality comparisons.
        Args:
           other: Other object to compare to
        Returns:
            -1/0/1 depending on whether self is ``<``, ``=`` or ``>`` other
        """
        if other is None:
            return 1
        if self.split != other.split:
            if self.split < other.split:
                return -1
            else:
                return 1
        return 0

    def _setSplit(self, value):
        """
        Property target used to set the split configuration value.
        If not ``None``, the value must be a ``SplitConfig`` object.
        Raises:
           ValueError: If the value is not a ``SplitConfig``
        """
        if value is None:
            self._split = None
        else:
            if not isinstance(value, SplitConfig):
                raise ValueError("Value must be a ``SplitConfig`` object.")
            self._split = value

    def _getSplit(self):
        """
        Property target used to get the split configuration value.
        """
        return self._split

    split = property(_getSplit, _setSplit, None, "Split configuration in terms of a ``SplitConfig`` object.")

    def validate(self):
        """
        Validates configuration represented by the object.

        Split configuration must be filled in.  Within that, both the size limit
        and split size must be filled in.

        Raises:
           ValueError: If one of the validations fails
        """
        if self.split is None:
            raise ValueError("Split section is required.")
        if self.split.sizeLimit is None:
            raise ValueError("Size limit must be set.")
        if self.split.splitSize is None:
            raise ValueError("Split size must be set.")

    def addConfig(self, xmlDom, parentNode):
        """
        Adds a <split> configuration section as the next child of a parent.

        Third parties should use this function to write configuration related to
        this extension.

        We add the following fields to the document::

           sizeLimit      //cb_config/split/size_limit
           splitSize      //cb_config/split/split_size

        Args:
           xmlDom: DOM tree as from ``impl.createDocument()``
           parentNode: Parent that the section should be appended to
        """
        if self.split is not None:
            sectionNode = addContainerNode(xmlDom, parentNode, "split")
            addByteQuantityNode(xmlDom, sectionNode, "size_limit", self.split.sizeLimit)
            addByteQuantityNode(xmlDom, sectionNode, "split_size", self.split.splitSize)

    def _parseXmlData(self, xmlData):
        """
        Internal method to parse an XML string into the object.

        This method parses the XML document into a DOM tree (``xmlDom``) and then
        calls a static method to parse the split configuration section.

        Args:
           xmlData (String data): XML data to be parsed
        Raises:
           ValueError: If the XML cannot be successfully parsed
        """
        (xmlDom, parentNode) = createInputDom(xmlData)
        self._split = LocalConfig._parseSplit(parentNode)

    @staticmethod
    def _parseSplit(parent):
        """
        Parses an split configuration section.

        We read the following individual fields::

           sizeLimit      //cb_config/split/size_limit
           splitSize      //cb_config/split/split_size

        Args:
           parent: Parent node to search beneath

        Returns:
            ``EncryptConfig`` object or ``None`` if the section does not exist
        Raises:
           ValueError: If some filled-in value is invalid
        """
        split = None
        section = readFirstChild(parent, "split")
        if section is not None:
            split = SplitConfig()
            split.sizeLimit = readByteQuantity(section, "size_limit")
            split.splitSize = readByteQuantity(section, "split_size")
        return split


########################################################################
# Public functions
########################################################################

###########################
# executeAction() function
###########################


# pylint: disable=W0613
def executeAction(configPath, options, config):
    """
    Executes the split backup action.

    Args:
       configPath (String representing a path on disk): Path to configuration file on disk
       options (Options object): Program command-line options
       config (Config object): Program configuration
    Raises:
       ValueError: Under many generic error conditions
       IOError: If there are I/O problems reading or writing files
    """
    logger.debug("Executing split extended action.")
    if config.options is None or config.stage is None:
        raise ValueError("Cedar Backup configuration is not properly filled in.")
    local = LocalConfig(xmlPath=configPath)
    dailyDirs = findDailyDirs(config.stage.targetDir, SPLIT_INDICATOR)
    for dailyDir in dailyDirs:
        _splitDailyDir(
            dailyDir, local.split.sizeLimit, local.split.splitSize, config.options.backupUser, config.options.backupGroup
        )
        writeIndicatorFile(dailyDir, SPLIT_INDICATOR, config.options.backupUser, config.options.backupGroup)
    logger.info("Executed the split extended action successfully.")


##############################
# _splitDailyDir() function
##############################


def _splitDailyDir(dailyDir, sizeLimit, splitSize, backupUser, backupGroup):
    """
    Splits large files in a daily staging directory.

    Files that match INDICATOR_PATTERNS (i.e. ``"cback.store"``,
    ``"cback.stage"``, etc.) are assumed to be indicator files and are ignored.
    All other files are split.

    Args:
       dailyDir: Daily directory to encrypt
       sizeLimit: Size limit, in bytes
       splitSize: Split size, in bytes
       backupUser: User that target files should be owned by
       backupGroup: Group that target files should be owned by

    Raises:
       ValueError: If the encrypt mode is not supported
       ValueError: If the daily staging directory does not exist
    """
    logger.debug("Begin splitting contents of [%s].", dailyDir)
    fileList = getBackupFiles(dailyDir)  # ignores indicator files
    for path in fileList:
        size = float(os.stat(path).st_size)
        if size > sizeLimit:
            _splitFile(path, splitSize, backupUser, backupGroup, removeSource=True)
    logger.debug("Completed splitting contents of [%s].", dailyDir)


########################
# _splitFile() function
########################


def _splitFile(sourcePath, splitSize, backupUser, backupGroup, removeSource=False):
    """
    Splits the source file into chunks of the indicated size.

    The split files will be owned by the indicated backup user and group.  If
    ``removeSource`` is ``True``, then the source file will be removed after it is
    successfully split.

    Args:
       sourcePath: Absolute path of the source file to split
       splitSize: Encryption mode (only "gpg" is allowed)
       backupUser: User that target files should be owned by
       backupGroup: Group that target files should be owned by
       removeSource: Indicates whether to remove the source file

    Raises:
       IOError: If there is a problem accessing, splitting or removing the source file
    """
    cwd = os.getcwd()
    try:
        if not os.path.exists(sourcePath):
            raise ValueError("Source path [%s] does not exist." % sourcePath)
        dirname = os.path.dirname(sourcePath)
        filename = os.path.basename(sourcePath)
        prefix = "%s_" % filename
        bytes = int(splitSize.bytes)  # pylint: disable=W0622
        os.chdir(dirname)  # need to operate from directory that we want files written to
        command = resolveCommand(SPLIT_COMMAND)
        args = ["--verbose", "--numeric-suffixes", "--suffix-length=5", "--bytes=%d" % bytes, filename, prefix]
        (result, output) = executeCommand(command, args, returnOutput=True, ignoreStderr=False)
        if result != 0:
            raise IOError("Error [%d] calling split for [%s]." % (result, sourcePath))
        pattern = re.compile(r"(creating file [`'])(%s)(.*)(')" % prefix)
        match = pattern.search(output[-1:][0])
        if match is None:
            raise IOError("Unable to parse output from split command.")
        value = int(match.group(3).strip())
        for index in range(0, value):
            path = "%s%05d" % (prefix, index)
            if not os.path.exists(path):
                raise IOError("After call to split, expected file [%s] does not exist." % path)
            changeOwnership(path, backupUser, backupGroup)
        if removeSource:
            if os.path.exists(sourcePath):
                try:
                    os.remove(sourcePath)
                    logger.debug("Completed removing old file [%s].", sourcePath)
                except:
                    raise IOError("Failed to remove file [%s] after splitting it." % (sourcePath))
    finally:
        os.chdir(cwd)



# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Copyright (c) 2007,2010,2015 Kenneth J. Pronovici.
# All rights reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# Version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# Copies of the GNU General Public License are available from
# the Free Software Foundation website, http://www.gnu.org/.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Cedar Backup, release 3
# Purpose  : Tests action utility functionality.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Unit tests for CedarBackup3/actions/util.py.

Code Coverage
=============

   This module contains individual tests for the public functions and classes
   implemented in actions/util.py.

Naming Conventions
==================

   I prefer to avoid large unit tests which validate more than one piece of
   functionality, and I prefer to avoid using overly descriptive (read: long)
   test names, as well.  Instead, I use lots of very small tests that each
   validate one specific thing.  These small tests are then named with an index
   number, yielding something like ``testAddDir_001`` or ``testValidate_010``.
   Each method has a docstring describing what it's supposed to accomplish.  I
   feel that this makes it easier to judge how important a given failure is,
   and also makes it somewhat easier to diagnose and fix individual problems.

Full vs. Reduced Tests
======================

   All of the tests in this module are considered safe to be run in an average
   build environment.  There is a no need to use a ACTIONSUTILTESTS_FULL
   environment variable to provide a "reduced feature set" test suite as for
   some of the other test modules.

@author Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Import modules and do runtime validations
########################################################################

import os
import tempfile
import unittest

from CedarBackup3.actions.util import findDailyDirs, writeIndicatorFile
from CedarBackup3.extend.encrypt import ENCRYPT_INDICATOR
from CedarBackup3.testutil import buildPath, configureLogging, extractTar, findResources, removedir

#######################################################################
# Module-wide configuration and constants
#######################################################################

DATA_DIRS = [
    "./data",
    "./tests/data",
]
RESOURCES = [
    "tree1.tar.gz",
    "tree8.tar.gz",
    "tree15.tar.gz",
    "tree17.tar.gz",
    "tree18.tar.gz",
    "tree19.tar.gz",
    "tree20.tar.gz",
]

INVALID_PATH = "bogus"  # This path name should never exist


#######################################################################
# Test Case Classes
#######################################################################

######################
# TestFunctions class
######################


class TestFunctions(unittest.TestCase):
    """Tests for the various public functions."""

    ################
    # Setup methods
    ################

    @classmethod
    def setUpClass(cls):
        configureLogging()

    def setUp(self):
        try:
            self.tmpdir = tempfile.mkdtemp()
            self.resources = findResources(RESOURCES, DATA_DIRS)
        except Exception as e:
            self.fail(e)

    def tearDown(self):
        try:
            removedir(self.tmpdir)
        except:
            pass

    ##################
    # Utility methods
    ##################

    def extractTar(self, tarname):
        """Extracts a tarfile with a particular name."""
        extractTar(self.tmpdir, self.resources["%s.tar.gz" % tarname])

    def buildPath(self, components):
        """Builds a complete search path from a list of components."""
        components.insert(0, self.tmpdir)
        return buildPath(components)

    #######################
    # Test findDailyDirs()
    #######################

    def testFindDailyDirs_001(self):
        """
        Test with a nonexistent staging directory.
        """
        stagingDir = self.buildPath([INVALID_PATH])
        self.assertRaises(ValueError, findDailyDirs, stagingDir, ENCRYPT_INDICATOR)

    def testFindDailyDirs_002(self):
        """
        Test with an empty staging directory.
        """
        self.extractTar("tree8")
        stagingDir = self.buildPath(["tree8", "dir001"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual([], dailyDirs)

    def testFindDailyDirs_003(self):
        """
        Test with a staging directory containing only files.
        """
        self.extractTar("tree1")
        stagingDir = self.buildPath(["tree1"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual([], dailyDirs)

    def testFindDailyDirs_004(self):
        """
        Test with a staging directory containing only links.
        """
        self.extractTar("tree15")
        stagingDir = self.buildPath(["tree15", "dir001"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual([], dailyDirs)

    def testFindDailyDirs_005(self):
        """
        Test with a valid staging directory, where the daily directories do NOT
        contain the encrypt indicator.
        """
        self.extractTar("tree17")
        stagingDir = self.buildPath(["tree17"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual(6, len(dailyDirs))
        self.assertTrue(self.buildPath(["tree17", "2006", "12", "29"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree17", "2006", "12", "30"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree17", "2006", "12", "31"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree17", "2007", "01", "01"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree17", "2007", "01", "02"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree17", "2007", "01", "03"]) in dailyDirs)

    def testFindDailyDirs_006(self):
        """
        Test with a valid staging directory, where the daily directories DO
        contain the encrypt indicator.
        """
        self.extractTar("tree18")
        stagingDir = self.buildPath(["tree18"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual([], dailyDirs)

    def testFindDailyDirs_007(self):
        """
        Test with a valid staging directory, where some daily directories contain
        the encrypt indicator and others do not.
        """
        self.extractTar("tree19")
        stagingDir = self.buildPath(["tree19"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual(3, len(dailyDirs))
        self.assertTrue(self.buildPath(["tree19", "2006", "12", "30"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree19", "2007", "01", "01"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree19", "2007", "01", "03"]) in dailyDirs)

    def testFindDailyDirs_008(self):
        """
        Test for case where directories other than daily directories contain the
        encrypt indicator (the indicator should be ignored).
        """
        self.extractTar("tree20")
        stagingDir = self.buildPath(["tree20"])
        dailyDirs = findDailyDirs(stagingDir, ENCRYPT_INDICATOR)
        self.assertEqual(6, len(dailyDirs))
        self.assertTrue(self.buildPath(["tree20", "2006", "12", "29"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree20", "2006", "12", "30"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree20", "2006", "12", "31"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree20", "2007", "01", "01"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree20", "2007", "01", "02"]) in dailyDirs)
        self.assertTrue(self.buildPath(["tree20", "2007", "01", "03"]) in dailyDirs)

    ############################
    # Test writeIndicatorFile()
    ############################

    def testWriteIndicatorFile_001(self):
        """
        Test with a nonexistent staging directory.
        """
        stagingDir = self.buildPath([INVALID_PATH])
        self.assertRaises(IOError, writeIndicatorFile, stagingDir, ENCRYPT_INDICATOR, None, None)

    def testWriteIndicatorFile_002(self):
        """
        Test with a valid staging directory.
        """
        self.extractTar("tree8")
        stagingDir = self.buildPath(["tree8", "dir001"])
        writeIndicatorFile(stagingDir, ENCRYPT_INDICATOR, None, None)
        self.assertTrue(os.path.exists(self.buildPath(["tree8", "dir001", ENCRYPT_INDICATOR])))



# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Cedar Backup, release 3
# Purpose  : Provides interface backwards compatibility.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Provides interface backwards compatibility.

In Cedar Backup 2.10.0, a refactoring effort took place while adding code to
support DVD hardware.  All of the writer functionality was moved to the
writers/ package.  This mostly-empty file remains to preserve the Cedar Backup
library interface.

:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""

########################################################################
# Imported modules
########################################################################

from CedarBackup3.writers.cdwriter import (  # pylint: disable=W0611
    MEDIA_CDR_74,
    MEDIA_CDR_80,
    MEDIA_CDRW_74,
    MEDIA_CDRW_80,
    CdWriter,
    MediaCapacity,
    MediaDefinition,
)
from CedarBackup3.writers.util import validateDriveSpeed, validateScsiId  # pylint: disable=W0611



# -*- coding: utf-8 -*-
# vim: set ft=python ts=4 sw=4 expandtab:
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
#              C E D A R
#          S O L U T I O N S       "Software done right."
#           S O F T W A R E
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Author   : Kenneth J. Pronovici <pronovic@ieee.org>
# Language : Python 3
# Project  : Official Cedar Backup Extensions
# Purpose  : Provides package initialization
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

########################################################################
# Module documentation
########################################################################

"""
Cedar Backup writers.

This package consolidates all of the modules that implenent "image writer"
functionality, including utilities and specific writer implementations.

:author: Kenneth J. Pronovici <pronovic@ieee.org>
"""


########################################################################
# Package initialization
########################################################################

# Using 'from CedarBackup3.writers import *' will just import the modules listed
# in the __all__ variable.

import CedarBackup3.writers.cdwriter
import CedarBackup3.writers.dvdwriter
import CedarBackup3.writers.util

__all__ = ["util", "cdwriter", "dvdwriter"]
